@@ -28,7 +28,7 @@
  *
  */
 
-static const char revision[] = "$Id: qslog.c,v 1.98 2015/09/21 20:47:13 pbuchbinder Exp $";
+static const char revision[] = "$Id: qslog.c,v 1.102 2016/05/25 11:39:58 pbuchbinder Exp $";
 
 #include <stdio.h>
 #include <string.h>
@@ -156,6 +156,8 @@ typedef struct stat_rec_st {
   long qos_t;
   long qos_l;
   long qos_ser;
+  long qos_a;
+  long qos_u;
 
   apr_table_t *events;
   apr_pool_t *pool;
@@ -189,8 +191,9 @@ static char  m_file_name2[MAX_LINE];
 static int   m_rotate = 0;
 static int   m_generations = QS_GENERATIONS;
 /* regex to search the time string */
-static regex_t m_trx;
-static regex_t m_trx2;
+static regex_t m_trx_access;
+static regex_t m_trx_j;
+static regex_t m_trx_g;
 /* real time mode (default) or offline */
 static int   m_off = 0;
 static int   m_offline = 0;
@@ -210,6 +213,8 @@ static int   m_methods = 0;
 /* debug/offline */
 static long  m_lines = 0;
 static int   m_verbose = 0;
+/* enable/disable event counter */
+static int   m_hasEV = 0;
 
 /* events ----------------------------------------------------- */
 /*
@@ -716,14 +721,6 @@ static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec,
           ">5s;%ld;"
           "%s"
           "%s"
-          "qV;%ld;"
-          "qS;%ld;"
-          "qD;%ld;"
-          "qK;%ld;"
-          "qT;%ld;"
-          "qL;%ld;"
-          "qs;%ld;"
-          "%s"
           ,
           timeStr,
           main ? "" : stat_rec->id,
@@ -747,16 +744,31 @@ static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec,
           stat_rec->duration_5,
           stat_rec->duration_6,
           ip,
-          usr,
-          stat_rec->qos_v,
-          stat_rec->qos_s,
-          stat_rec->qos_d,
-          stat_rec->qos_k,
-          stat_rec->qos_t,
-          stat_rec->qos_l,
-          stat_rec->qos_ser,
-          custom
+          usr
           );
+  if(m_hasEV) {
+    fprintf(f,
+            "qV;%ld;"
+            "qS;%ld;"
+            "qD;%ld;"
+            "qK;%ld;"
+            "qT;%ld;"
+            "qL;%ld;"
+            "qs;%ld;"
+            "qA;%ld;"
+            "qu;%ld;",
+            stat_rec->qos_v,
+            stat_rec->qos_s,
+            stat_rec->qos_d,
+            stat_rec->qos_k,
+            stat_rec->qos_t,
+            stat_rec->qos_l,
+            stat_rec->qos_ser,
+            stat_rec->qos_a,
+            stat_rec->qos_u);
+  }
+  fprintf(f, "%s",
+          custom);
   stat_rec->line_count = 0;
   stat_rec->byte_count = 0;
   if(stat_rec->i_byte_count != -1) {
@@ -791,6 +803,8 @@ static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec,
   stat_rec->qos_t = 0;
   stat_rec->qos_l = 0;
   stat_rec->qos_ser = 0;
+  stat_rec->qos_a = 0;
+  stat_rec->qos_u = 0;
   if(main) {
     if(!offline) {
       fprintf(f, "sl;%.2f;", av[0]);
@@ -936,6 +950,8 @@ static stat_rec_t *createRec(apr_pool_t *pool, const char *id, const char *patte
   rec->qos_t = 0;
   rec->qos_l = 0;
   rec->qos_ser = 0;
+  rec->qos_a = 0;
+  rec->qos_u = 0;
 
   rec->events = apr_table_make(pool, 300);
   rec->pool = pool;
@@ -1242,6 +1258,12 @@ static void updateRec(stat_rec_t *rec, char *T, char *t, char *D, char *S,
     if(strchr(Q, 's') != NULL) {
       rec->qos_ser++;
     }
+    if(strchr(Q, 'A') != NULL) {
+      rec->qos_a++;
+    }
+    if(strchr(Q, 'u') != NULL) {
+      rec->qos_u++;
+    }
   }
   if(E != NULL) {
     qs_updateEvents(rec->pool, E, rec->events);
@@ -1545,85 +1567,112 @@ static int mstr2i(const char *m) {
   return 0;
 }
 
+/**
+ * Extracts the time from the log line using the 
+ * Apache access default time format (%t)
+ */
+static time_t getMinutesAccessLog(char *line, regmatch_t ma) {
+  time_t minutes = 0;
+  int buf_len = ma.rm_eo - ma.rm_so + 1;
+  char buf[buf_len];
+  strncpy(buf, &line[ma.rm_so], ma.rm_eo - ma.rm_so);
+  buf[ma.rm_eo - ma.rm_so] = '\0';
+  /* dd/MMM/yyyy:hh:mm:ss */
+  /* cut seconds */
+  buf[strlen(buf)-3] = '\0';
+  /* get minutes */
+  minutes = minutes + (atoi(&buf[strlen(buf)-2]));
+  /* cut minutes */
+  buf[strlen(buf)-3] = '\0';
+  /* get hours */
+  minutes = minutes + (atoi(&buf[strlen(buf)-2]) * 60);
+  
+  /* store date information */
+  {
+    char *year;
+    char *month;
+    char *day;
+    /* cut hours */
+    buf[strlen(buf)-3] = '\0';
+    year = &buf[strlen(buf)-4];
+    /* cut year */
+    buf[strlen(buf)-5] = '\0';
+    month = &buf[strlen(buf)-3];
+    /* cut month */
+    buf[strlen(buf)-4] = '\0';
+    day = buf;
+    snprintf(m_date_str, sizeof(m_date_str), "%s.%02d.%s", day, mstr2i(month), year);
+  }
+  return minutes;
+}
+
+/**
+ * Extracts the time from the log line using the 
+ * the time patterns "yyyy mm dd hh:mm:ss,mmm" or
+ * "yyyy mm dd hh:mm:ss.mmm"
+ */
+static time_t getMinutesJLog(char *line, regmatch_t ma) {
+  time_t minutes = 0;
+  int buf_len = ma.rm_eo - ma.rm_so + 1;
+  char buf[buf_len];
+  strncpy(buf, &line[ma.rm_so], ma.rm_eo - ma.rm_so);
+  buf[ma.rm_eo - ma.rm_so] = '\0';
+  /* yyyy mm dd hh:mm:ss,mmm */
+  /* cut seconds */
+  buf[strlen(buf)-7] = '\0';
+  /* get minutes */
+  minutes = minutes + (atoi(&buf[strlen(buf)-2]));
+  /* cut minutes */
+  buf[strlen(buf)-3] = '\0';
+  /* get hours */
+  minutes = minutes + (atoi(&buf[strlen(buf)-2]) * 60);
+  /* store date information */
+  {
+    char *year;
+    char *month;
+    char *day;
+    /* cut hours */
+    buf[strlen(buf)-3] = '\0';
+    day = &buf[strlen(buf)-2];
+    /* cut day */
+    buf[strlen(buf)-3] = '\0';
+    month = &buf[strlen(buf)-2];
+    /* cut month */
+    buf[strlen(buf)-3] = '\0';
+    year = buf;
+    snprintf(m_date_str, sizeof(m_date_str), "%s.%s.%s", day, month, year);
+  }
+  return minutes;
+}
+
 /*
- * get the time in minutes from the access log line
+ * gets today's time in minutes from the access log line
  */
 static time_t getMinutes(char *line) {
-  regmatch_t ma;
-  if(regexec(&m_trx, line, 1, &ma, 0) != 0) {
-    if(regexec(&m_trx2, line, 1, &ma, 0) == 0) {
-      time_t minutes = 0;
-      int buf_len = ma.rm_eo - ma.rm_so + 1;
-      char buf[buf_len];
-      strncpy(buf, &line[ma.rm_so], ma.rm_eo - ma.rm_so);
-      buf[ma.rm_eo - ma.rm_so] = '\0';
-      /* yyyy mm dd hh:mm:ss,mmm */
-      /* cut seconds */
-      buf[strlen(buf)-7] = '\0';
-      /* get minutes */
-      minutes = minutes + (atoi(&buf[strlen(buf)-2]));
-      /* cut minutes */
-      buf[strlen(buf)-3] = '\0';
-      /* get hours */
-      minutes = minutes + (atoi(&buf[strlen(buf)-2]) * 60);
-      /* store date information */
-      {
-        char *year;
-        char *month;
-        char *day;
-        /* cut hours */
-        buf[strlen(buf)-3] = '\0';
-        day = &buf[strlen(buf)-2];
-        /* cut day */
-        buf[strlen(buf)-3] = '\0';
-        month = &buf[strlen(buf)-2];
-        /* cut month */
-        buf[strlen(buf)-3] = '\0';
-        year = buf;
-        snprintf(m_date_str, sizeof(m_date_str), "%s.%s.%s", day, month, year);
-      }
-      return minutes;
-    } else {
-      // unknown format (not relevant for -pu/-puc
-      if(m_offline_url == 0) {
-        fprintf(stdout, "F(%ld)", m_lines);
-      }
-      return 0;
-    }
-  } else {
+  regmatch_t ma[2];
+  if(regexec(&m_trx_access, line, 1, ma, 0) == 0) {
+    return getMinutesAccessLog(line, ma[0]);
+  }
+  if(regexec(&m_trx_j, line, 1, ma, 0) == 0) {
+    return getMinutesJLog(line, ma[0]);
+  }
+  if(regexec(&m_trx_g, line, 2, ma, 0) == 0) {
     time_t minutes = 0;
-    int buf_len = ma.rm_eo - ma.rm_so + 1;
-    char buf[buf_len];
-    strncpy(buf, &line[ma.rm_so], ma.rm_eo - ma.rm_so);
-    buf[ma.rm_eo - ma.rm_so] = '\0';
-    /* dd/MMM/yyyy:hh:mm:ss */
-    /* cut seconds */
-    buf[strlen(buf)-3] = '\0';
-    /* get minutes */
-    minutes = minutes + (atoi(&buf[strlen(buf)-2]));
-    /* cut minutes */
-    buf[strlen(buf)-3] = '\0';
-    /* get hours */
-    minutes = minutes + (atoi(&buf[strlen(buf)-2]) * 60);
-
-    /* store date information */
-    {
-      char *year;
-      char *month;
-      char *day;
-      /* cut hours */
-      buf[strlen(buf)-3] = '\0';
-      year = &buf[strlen(buf)-4];
-      /* cut year */
-      buf[strlen(buf)-5] = '\0';
-      month = &buf[strlen(buf)-3];
-      /* cut month */
-      buf[strlen(buf)-4] = '\0';
-      day = buf;
-      snprintf(m_date_str, sizeof(m_date_str), "%s.%02d.%s", day, mstr2i(month), year);
-    }
+    int len = ma[1].rm_eo - ma[1].rm_so;
+    char buf[len+1];
+    strncpy(buf, &line[ma[1].rm_so], len);
+    buf[len] = '\0';
+    /* hh:mm */
+    buf[2] = '\0';
+    minutes = atoi(buf) * 60;
+    minutes = minutes + atoi(&buf[3]);
     return minutes;
   }
+  // unknown format (not relevant for "-pu"/"-puc" but for "-p" mode)
+  if(m_offline_url == 0) {
+    fprintf(stdout, "F(%ld)", m_lines);
+  }
+  return 0;
 }
 
 /*
@@ -1689,16 +1738,18 @@ static void readStdinOffline(apr_pool_t *pool, const char *cstr) {
     } else {
       if(l_time > unitTime) {
         if(!m_verbose) {
-          fprintf(outdev, ".");
-          fflush(outdev);
+          if(m_f != stdout) {
+            fprintf(outdev, ".");
+            fflush(outdev);
+          }
         }
       }
       while(l_time > unitTime) {
+        unitTime++;
         snprintf(buf, sizeof(buf), "%s %.2ld:%.2ld:00", m_date_str, unitTime/60, unitTime%60);
         if(m_offline) {
           printAndResetStat(buf);
         }
-        unitTime++;
         qs_setTime(unitTime * 60);;
       }
       updateStat(pool, cstr, line);
@@ -1803,7 +1854,8 @@ static void usage(const char *cmd, int man) {
   qs_man_println(man, "  - number of events identified by the 'E' format character\n");
   qs_man_println(man, "  - number of mod_qos events within the last minute (qV=create session,\n");
   qs_man_print(man, "    qS=session pass, qD=access denied, qK=connection closed, qT=dynamic\n");
-  qs_man_print(man, "    keep-alive, qL=request/response slow down, qs=serialized request)\n");
+  qs_man_print(man, "    keep-alive, qL=request/response slow down, qs=serialized request, \n");
+  qs_man_print(man, "    qA=connection abort, qU=new user tracking cookie)\n");
   printf("\n");
   if(man) {
     printf(".SH OPTIONS\n");
@@ -2018,6 +2070,9 @@ int main(int argc, const char *const argv[]) {
           // enable custom counter
           m_customcounter = 1;
         }
+        if(strchr(config, 'Q')) {
+          m_hasEV = 1;
+        }
       }
     } else if(strcmp(*argv,"-o") == 0) { /* this is the out file */
       if (--argc >= 1) {
@@ -2077,14 +2132,19 @@ int main(int argc, const char *const argv[]) {
     if(nice(10) == -1) {
       fprintf(stderr, "ERROR, failed to change nice value: %s\n", strerror(errno));
     }
-    /* init time pattern regex, std apache access log */
-    regcomp(&m_trx, 
+    /* init time pattern regex, std apache access log "dd/MMM/yyyy:hh:mm:ss" */
+    regcomp(&m_trx_access, 
             "[0-9]{2}/[a-zA-Z]{3}/[0-9]{4}:[0-9]{2}:[0-9]{2}:[0-9]{2}",
             REG_EXTENDED);
-    /* other time patterns: "yyyy mm dd hh:mm:ss,mmm" or "yyyy mm dd hh:mm:ss.mmm" */
-    regcomp(&m_trx2, 
+    /* other time patterns: "yyyy mm dd hh:mm:ss,mmm" or "yyyy mm dd hh:mm:ss.mmm"
+       resp  "yyyy-mm-dd hh:mm:ss,mmm" or "yyyy-mm-dd hh:mm:ss.mmm" */
+    regcomp(&m_trx_j,
             "[0-9]{4}[ -]{1}[0-9]{2}[ -]{1}[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}[,.]{1}[0-9]{3}",
             REG_EXTENDED);
+    /* fallback to generic " hh:mm:ss " pattern */
+    regcomp(&m_trx_g,
+            " ([0-9]{2}:[0-9]{2}):[0-9]{2} ",
+            REG_EXTENDED);
   }
 
   /*
